
DEAD = 1
ALIVE = 2

function wait(n) 
	for i = 0, n*100000 do io.write("") end -- do nothing 100.000 times
end

function createObserverMap (cells, legend)
	Observer {
		subject     = cells,
		attributes  = cells.attribute,
		legends     = {legend},
		type = "map"
	}
end

----
function createMap (model)

	createObserverMap(model.cells, model.legend)
	model.cells:notify(0) 

end
------------------
function createTimer (model)

	model.timer = Timer {
		Event{time = 1, period = 1, action = function(e)	
			model.cells:synchronize()
			rules(model)    -- 
			model.cells:notify(e:getTime())
			wait (10) 
		end}
	}
end

function count_neighbors(cell)
	count = 0
	forEachNeighbor(cell, function(cell, neighbor)
		if neighbor.past.color == ALIVE then
			count = count + 1
		end
	end)
	return count
end

-------------------

function fill_glider(cs)
	forEachCell(cs, function(cell)
		cell.color = DEAD
	end)

	cs:getCell(Coord{x=3,y=3}).color = ALIVE
	cs:getCell(Coord{x=4,y=3}).color = ALIVE
	cs:getCell(Coord{x=5,y=3}).color = ALIVE
	cs:getCell(Coord{x=5,y=2}).color = ALIVE
	cs:getCell(Coord{x=4,y=1}).color = ALIVE
end

function rules (model)

	forEachCell(model.cells, function(cell)
		neighbors = count_neighbors(cell)
		if neighbors < 2 then
			cell.color = DEAD
		elseif neighbors > 3 then
			cell.color = DEAD
		elseif neighbors == 3 and cell.past.color == DEAD then
			cell.color = ALIVE
		end
	end)
end

function init (model)

	fill_glider (model.cells)

	model.cells:createNeighborhood{
		strategy = "moore",
		self = false,
		wrap = true
	}
end

-----------------

function run (model, time)
	model.timer:execute (time)
end

-----------
function CellularAutomataModel(model)

	init (model)

	-- create Plot
	createMap (model)

	-- timer
	createTimer (model)

	return model
end
---------
life =  CellularAutomataModel {
	cells = CellularSpace{ 
		xdim = 30, 
		ydim = 30,
		attribute = {"color"}
	},
	legend = Legend {
		grouping = "uniquevalue",
		colorBar = {
			{value = DEAD, color = "white"},
			{value = ALIVE, color = "black"}
		}
	}
}

-----
run (life, 400) 




